@@ -6,7 +6,7 @@ DOCSIS::ConfigFile - Decodes and encodes DOCSIS config-files
 
 =head1 VERSION
 
-Version 0.59
+0.5901
 
 =head1 SYNOPSIS
 
@@ -41,7 +41,7 @@ use constant Syminfo => "DOCSIS::ConfigFile::Syminfo";
 use constant Decode  => "DOCSIS::ConfigFile::Decode";
 use constant Encode  => "DOCSIS::ConfigFile::Encode";
 
-our $VERSION = '0.59';
+our $VERSION = '0.5901';
 our $TRACE   = 0;
 
 =head1 METHODS
